/*
	fpcalc.c - a minimalist's Floating Piont Calculator of Ant_Anl
	Copyright (C) 2005 Vadim Demidov vadim_d .at. sourceforge.net

    Ant_Anl is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published
    by the Free Software Foundation; either version 2 of the License,
    or (at your option) any later version.

    Ant_Anl is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ant_Anl; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Since a free edition of B Knudesn Data CC5X compiler allows only
	1K of code per object module, all the floating point math (four
	basic arithmetic operations and square root) must fit into such
	a module. One more limitation is all intermodule function calls
	are limited to one 8-bit parameter and one 8-bit return value.
	To handle this, two floating point registers are made global,
	and a single global fuction receives the only parameter telling
	what to do. Beside five math operations mentioned above, it also
	converts a 16-bit unsigned value to floating point (input), and
	a floating point value to signed 24-bit integer (output). It's
	important that the Math library is called by a single external
	function - otherwise you get an error message.

	Revision History:

	17 Apr 05 - Created (VD)
	27 Apr 05 - FPC_INITF added to FP opcodes, FPC_NEG removed (VD)
*/

#include "aa_defs.h"	// Definitions common to all Ant_Anl project
#include "glb_defs.h"	// Global functions and data objects

#pragma rambank 1
#pragma codepage 1

#define FP_OPTIM_SIZE   // optimize for SIZE - the only way to fit!
//#define DISABLE_ROUNDING   // disable rounding and save code space

#include <math24f.h>
#include <math24lb.h>

float24 fp_x, fp_y;	// Global variables

void fpcalc(char W)

{
	switch (W) {			// See the operation required
		case FPC_INITF:
			InitFpFlags();	// Initialize FP flags (sets rounding)
			break;
		case FPC_UP:
			fp_y=fp_x;		// X -> Y
			break;
		case FPC_U16TOFP:
			fp_x=fp_x_u16;	// 16-bit unsigned to FP
			break;
		case FPC_FPTOS24:
			fp_x_i24=fp_x;	// FP to signed 24-bit
			break;
		case FPC_ADD:
			fp_x+=fp_y;		// X := Y+X
			break;
		case FPC_SUB:
			fp_x=fp_y-fp_x;	// X := Y-X
			break;
		case FPC_MUL:
			fp_x*=fp_y;		// X := Y*X
			break;
		case FPC_DIV:
			fp_x=fp_y/fp_x;	// X := Y/X
			break;
		case FPC_SQRT:
			fp_x=sqrt(fp_x); // X := SQRT(X)
			break;
	}
}
